/* ==============================================================[ ap-lirc.c ]=
 * Control AlsaPlayer through LIRC.
 * Copyright (C) 2002 Hubert Chan <hubert@uhoreg.ca>
 *
 * ----------------------------------------------------------------( License )-
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * ------------------------------------------------------------( Description )-
 *
 * This program allows one to control AlsaPlayer using a remote control through
 * LIRC (Linux Infrared Remote Control).
 *
 * Lots of code ripped from irexec.c.  Commands based on lirc-xmms-plugin.
 *
 * ------------------------------------------------------------------------- */

#include <string.h>
#include <ctype.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include <alsaplayer/control.h>
#include <lirc/lirc_client.h>

const int ap_session = 0;

int main(int argc, char *argv[])
{
  struct lirc_config *config;

  char *progname=argv[0];
  if(argc>2)
  {
    fprintf(stderr,"Usage: %s <config file>\n",progname);
    exit(1);
  }

#ifdef DEBUG
  printf("Alsa Player version %d (whatever that means)\n", ap_version());
#endif

  if (!ap_session_running(ap_session))
  { /* if AlsaPlayer is not already running, start it */
#ifdef DEBUG
    printf("Warning: AlsaPlayer not running -- starting\n");
#endif
    if (system("alsaplayer&") < 0)
    {
      fprintf(stderr,"Error: could not start alsaplayer\n");
      exit(1);
    }
  }

  if(lirc_init("alsaplayer",1)==-1) exit(1);

  if(lirc_readconfig(argc==2 ? argv[1]:NULL,&config,NULL)==0)
  {
    char *code;
    char *c;
    char *args;
    int arg_int;
    int ret;

    while(lirc_nextcode(&code)==0)
    {
      if (!ap_ping(ap_session))
      {
	fprintf(stderr,"Error: AlsaPlayer died\n");
	free(code);
	lirc_freeconfig(config);
	lirc_deinit();
	exit(1);
      }

      if(code==NULL) continue;
      while((ret=lirc_code2char(config,code,&c))==0 &&
	    c!=NULL)
      {
	int i;
	for (i = 0; c[i] != '\0'; i++)
	  c[i] = tolower(c[i]);
#ifdef DEBUG
	printf("command: %s\n", c);
#endif

	c = strdup(c);
	/* separate command from arguments -- split at the first space */
	if ((args = strchr(c, ' ')) != NULL)
	{
	  *args = '\0';
	  args++;
	  while (*args == ' ')
	    args++;
	  arg_int = atoi(args);
	}

	/* big, huge, if-else !!! */
	if (strcmp(c, "play") == 0)
	  ap_play(ap_session);
	else if (strcmp(c, "stop") == 0)
	  ap_stop(ap_session);
	else if (strcmp(c, "pause") == 0)
	  ap_pause(ap_session);
	else if (strcmp(c, "playpause") == 0)
	{ /* unpause if it was already paused, pause otherwise */
	  float speed;
	  ap_get_speed(ap_session,&speed);
	  if (speed == 0.0)
	    ap_unpause(ap_session);
	  else
	    ap_pause(ap_session);
	}
	else if (strcmp(c, "next") == 0)
	{
	  if (args)
	    for (i = 0; i < arg_int; i++)
	      ap_next(ap_session);
	  else
	    ap_next(ap_session);
	}
	else if (strcmp(c, "prev") == 0)
	{
	  if (args)
	    for (i = 0; i < arg_int; i++)
	      ap_prev(ap_session);
	  else
	    ap_prev(ap_session);
	}
	else if (strcmp(c, "quit") == 0)
	{
	  ap_quit(ap_session);
	  free(code);
	  lirc_freeconfig(config);
	  lirc_deinit();
	  exit(0);
	}
	else if (strcmp(c, "playlist_clear") == 0)
	  ap_clear_playlist(ap_session);
	else if (strcmp(c, "fwd") == 0)
	{
	  if (!args)
	    arg_int = 1;
	  ap_set_position_relative(ap_session, arg_int);
	}
	else if (strcmp(c, "bwd") == 0)
	{
	  if (!args)
	    arg_int = 1;
	  ap_set_position_relative(ap_session, -arg_int);
	}
	else if (strcmp(c, "vol_up") == 0)
	{
	  int volume;
	  if (!args)
	    arg_int = 5;
	  ap_get_volume(ap_session, &volume);
	  volume += arg_int;
	  if (volume > 100)
	    volume = 100;
	  else if (volume < 0)
	    volume = 0;
	  ap_set_volume(ap_session, volume);
	}
	else if (strcmp(c, "vol_down") == 0)
	{
	  int volume;
	  if (!args)
	    arg_int = -5;
	  ap_get_volume(ap_session, &volume);
	  volume += arg_int;
	  if (volume > 100)
	    volume = 100;
	  else if (volume < 0)
	    volume = 0;
	  ap_set_volume(ap_session, volume);
	}
	else if (strcmp(c, "pan_left") == 0)
	{
	  int pan;
	  if (!args)
	    arg_int = 5;
	  ap_get_pan(ap_session, &pan);
	  pan -= arg_int;
	  if (pan > 100)
	    pan = 100;
	  else if (pan < -100)
	    pan = 0;
	  ap_set_volume(ap_session, pan);
	}
	else if (strcmp(c, "pan_right") == 0)
	{
	  int pan;
	  if (!args)
	    arg_int = 5;
	  ap_get_pan(ap_session, &pan);
	  pan += arg_int;
	  if (pan > 100)
	    pan = 100;
	  else if (pan < -100)
	    pan = 0;
	  ap_set_volume(ap_session, pan);
	}
	else if (strcmp(c, "pan_center") == 0)
	  ap_set_volume(ap_session, 0);
	else if (strcmp(c, "faster") == 0)
	{
	  float speed;
	  if (!args)
	    arg_int = 20;
	  ap_get_speed(ap_session, &speed);
	  speed += (double) arg_int / 100.0;
	  ap_set_speed(ap_session, speed);
	}
	else if (strcmp(c, "slower") == 0)
	{
	  float speed;
	  if (!args)
	    arg_int = 20;
	  ap_get_speed(ap_session, &speed);
	  speed -= (double) arg_int / 100.0;
	  ap_set_speed(ap_session, speed);
	}
	else if (strcmp(c, "speed") == 0)
	{
	  if (!args)
	    arg_int = 100;
	  ap_set_speed(ap_session, (double) arg_int / 100.0);
	}

	free(c);
      }

      free(code);
      if(ret==-1) break;
    }
    lirc_freeconfig(config);
  }

  lirc_deinit();
  return 0;
}

/* end of ap-lirc.c */
